#include <Servo.h> 

Servo elbow,gripper1,gripper2;

const int elbowPin = 3;
const int gripperPin1 = 6;
const int gripperPin2 = 9;
const int gyro1RefPin = A0;
const int gyro1OutZPin = A1;
const int gyro2RefPin = A2;
const int gyro2OutZPin = A3;
const int sensitivity = 1.1;

const int OPEN = 0,CLOSE = 1;
const int CLOSED = 1;

int speedElbow =  100; //-200 to 200
int speedGripper = 15; // °/15ms
int stateGripper1 = CLOSED,stateGripper2 = CLOSED; // states of the gripper (CLOSED or OPEN)
int state = CLOSED; // grippers initiallly closed
int pos = 0;
int timeStart = 0,actualTime = 0; 
int timeRotation = 600; //360ms
int counter=0;
double gyroRef1 = 0,gyroRef2 = 0;
double angleVelocity1 = 0,angleVelocity2 = 0; // angular velocity of gryroscope 1 and 2
double angle1 = 0,angle2 = 0;
int previousTime = 0,presentTime = 0; // used for integration (gyroscope)
int deltaT = 0;
int offset1 = 0,offset2 = 0;
int angle = 0;
int leg = 1;

void setup()
{
  // set the different pins used by the microcontrollers and the initial speed of the servos
  elbow.attach(elbowPin);
  elbow.writeMicroseconds(1550);
  gripper1.attach(gripperPin1);
  gripper2.attach(gripperPin2);
  
  gripper1.write(pos);
  
  Serial.begin(9600); 
  previousTime = micros();
  angleVelocity1 = analogRead(gyro1OutZPin); // pin which receive the speed of the gyroscope in volt
  gyroRef1 = analogRead(gyro1RefPin); // pin which receive the reference voltage of the gyroscope
  offset1 = angleVelocity1 - gyroRef1; // calculation of the offset
  angleVelocity2 = analogRead(gyro2OutZPin);
  gyroRef2 = analogRead(gyro2RefPin);
  offset2 = angleVelocity2 - gyroRef2;
}

void loop()
{
  if(counter<1)
  {
    angle=0; 
    if(counter%2==0) // if the counter is even(odd), the leg which moves is the leg 1(leg2)
      leg = 1;
    else
      leg = 2;
    gripper(leg,OPEN); // open the gripper
    while(angle<150) // wait until the angle has reached its final position
    {
      gyroscope();
      if(leg==1)
        angle = angle1;
      else
        angle = angle2;
    }
    gripper(leg,CLOSE); // close the gripper
    
    counter++;
  }
}

void gripper(int numGripper,int action) // function which open or close the gripper
{
  if(numGripper==1) // according to the gripper that we want to open/close, the state will be the state either the state of the gripper 1 or gripper2
    state = stateGripper1;
  else
  {
    state = stateGripper2;
    action = !action;
  }
    
  if(action==OPEN && state == CLOSED) // to open the gripper we have to ensure that the gripper is closed
  {
    for(pos = 0; pos < 190; pos += speedGripper)  // goes from 0 degrees to 190 degrees with a speed proportional to speedGripper
    {
      if(numGripper==1) 
        gripper1.write(pos);   // tell servo to go to position in variable 'pos'
      else
        gripper2.write(pos); 
      delay(15);                       // waits 15ms for the servo to reach the position 
    } 
    
    if(numGripper==1) // update the state
      stateGripper1 = OPEN;
    else
      stateGripper2 = OPEN;
  }
  
  if(action==CLOSE && state == OPEN) // to close the gripper we have to ensure that the gripper is open
  {  
    for(pos = 190; pos>=15; pos-=speedGripper) // goes from 190 degrees to 15 degrees with a speed proportional to speedGripper
    {                        
      if(numGripper == 1)      
        gripper1.write(pos);      // tell servo to go to position in variable 'pos' 
      else
        gripper2.write(pos);
      delay(15);                       // waits 15ms for the servo to reach the position 
    } 
    
    if(numGripper==1) // update the state
      stateGripper1 = CLOSED;
    else
      stateGripper2 = CLOSED;
  }
}

void rotation() // function which action the principal(elbow) servo
{
  timeStart = millis();
  actualTime = timeStart;
  elbow.writeMicroseconds(1550+speedElbow); // run the servo at a certain speed
  while((actualTime-timeStart)<timeRotation) // wait during a time equal to timeRotation
    actualTime = millis();
    
  if(speedElbow != 0) // set the speed to 0
  {
    speedElbow = 0;
    elbow.writeMicroseconds(1550+speedElbow);
  }
}

void gyroscope() // function which calculates the angles of the 2 gyroscope
{
  angleVelocity1 = 0;
  angleVelocity2 = 0;
  angleVelocity1 = analogRead(gyro1OutZPin); // read the speeds (in volt) returned by the gyroscopes
  angleVelocity2 = analogRead(gyro2OutZPin);
  gyroRef1 = analogRead(gyro1RefPin); // read the reference voltage returned by the gyroscopes
  gyroRef2 = analogRead(gyro2RefPin);
  angleVelocity1 = (angleVelocity1 - gyroRef1); // calculation of the real speed (in volt) by taking into account the reference 
  angleVelocity2 = (angleVelocity2 - gyroRef2);
  if(angleVelocity1 > 2*offset1 && angleVelocity1 < 0) // ATTENTION : offset < 0, we verify if the voltage (speed) is inferior to 2*offset(safety margin) then the gyroscope doesn't move -> speed = 0
    angleVelocity1 = 0;
  else // calculation of the speed by taking into account the offset
    angleVelocity1 = (angleVelocity1 - offset1)*5*1000/(1024*1.1);  
  if(angleVelocity2 > 2*offset2 && angleVelocity2 < 0)
    angleVelocity2 = 0;
  else
    angleVelocity2 = (angleVelocity2 - offset2)*5*1000/(1024*sensitivity);  // sensitivity = 1.1 (datasheet)
  presentTime = micros();
  deltaT = presentTime-previousTime; 
  previousTime = presentTime;
  angle1 = angle1 + angleVelocity1*deltaT/1000000; // integration to obtain the angles
  angle2 = angle2 + angleVelocity2*deltaT/1000000;
  Serial.println(angle1);
}

void acceleration(int initialSpeed,int finalSpeed,int increment) // function which allow to accelerate/decelerate the principal servo 
{
  int speed = initialSpeed; // the first speed is equal to the initial speed
  elbow.writeMicroseconds(1550+speed);
  
  while(speed != finalSpeed) // increase or decrease the speed until the final speed is reached
  {
    speed = speed + increment;
    elbow.writeMicroseconds(1550+speed);
    delay(15);
  }
  
  elbow.writeMicroseconds(1550);
}
